/* the serial port queue module. (spqm)
 *
 * version history:
 *
 *	jun 02 1999
 *		1.0 first public release.
 *
 *
 * written by: gene chipman <gchipman@erols.com> and houman alborzi <houman@cs.umd.edu>.
 * guest appearance by: jaime montemayor <monte@cs.umd.edu>.
 *
 * instructions:
 *
 *	* this module maintains a circular queue on the serial port. the queue is maintained at two levels.
 *		the interrupt driven assembly code enqueues the list. The c functions serialq_get_b() and 
 *		serialq_get_nb() dequeue elements. currently, spqm supports only an input queue.
 *
 *	* to activate the module, you will need to load the files listed below. (note that the file serial.c is
 *		included here for completeness. it exists separately on the handy board support pages.
 *
 *	* you can change the size of the queue. first, you must change the size of the array serialq[].
 *		next, you must change line 2 in the function serialq_init(). here is an example: suppose
 *		you want to increase the queue to have 100 elements. you first modify the global declaration
 *		so that it reads 'char serialq[100];' (disregard the single quote.)
 *		next, you go to serialq_init() and change line 2 to read 'sciq_stop =  1+(int) &serialq[99];'.
 *		that's it.
 *
 *	* for an example of how you would use the functions, refer to comm.c.
 *
 * required files:
 *
 *	sciint.icb
 *	serialq.c
 *
 *	serial.c 
 *		(this is the same file originally authored by randy sargent and/or
 *		 fred martin, subsequently modified by xavier chabot)
 *
 *
 */



#define SQ_OK        1
#define SQ_EMPTY     0
#define SQ_TIMEDOUT  -1
#define SQ_OVERFLOW -2


/*
 * global variables required by the serial port queue module
 *
 */


/* increase or decrease your queue size by changing this array. */
/* this number must also be changed in serialq_init() */

char serialq[32]; /* STEP 1 !! change queue size here !! */




/* the condition code of the serial queue.
 *   0 = all's well
 *   1 = queue overflow
 *
 */



/* this variable is defined as a global by the assembly code and is 
 * called sciq_of
 *
 */

/* timeout in milliseconds. */

long serialq_timeout = (long) 1000;





/*
 * initialization of the serial port queue.
 * ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 *
 * call this routine during your program's initialization phase.
 * you can also use it to reset and empty your queue.
 *
 * NOTE: if you change queue size, then you need to modify this
 *       code too.
 *
 */

int serialq_init()
{

	sciq_start = (int) &serialq[0];

	sciq_stop =  1+(int) &serialq[31]; /* STEP 2 !! change size of queue here !! */

	sciq_head = sciq_start;

	sciq_tail = sciq_start;

	sciq_init(0);

}


/*
 * dequeue an element. (two options)
 *
 * call serialq_get_nb to [non-blocking] dequeue the
 *   oldest element in the serial port queue.
 *
 * call serialq_get_b  to [blocking] dequeue the
 *   oldest element in the serial port queue.
 *
 * return codes:
 *
 *   1: successfully retrieved a byte from the queue.
 *   0: empty queue, nothing to read.
 *  -1: timed out during blocking get.
 *  -2: queue overflow.
 *
 */




int serialq_get_nb ( int *val )
{

	int success;

	irq_disable(0);

	if(sciq_of)
	{
		success = -2;
	}

	else if(sciq_head==sciq_tail)
	{
		success = 0;
	}

	else
	{
		read_val(val);
		success = 1;
	}

	irq_enable(0);
	return success;

}




int serialq_get_b ( int *val )
{

	long start_time;

	long last_time;

	int success = -1;

	irq_disable(0);

	if(sciq_of)
	{
		success = -2;
	}

	else if(sciq_head!=sciq_tail)
	{
		read_val(val);
		success = 1;
	}

	else

	{
		irq_enable(0);
		start_time = mseconds();

		last_time = start_time;
		while( (last_time < start_time + serialq_timeout) || (success!=1) )
		{
			msleep((long) 1);
			irq_disable(0);

			if(sciq_head!=sciq_tail)
			{
				read_val(val);
				success = 1;
			}

			irq_enable(0);
			last_time = mseconds();

		}

	}

	irq_disable(0);
	return success;

}




/*
 * peek at the next dequeue element from the queue without
 *   removing it. this is non-blocking.
 *
 * return codes: same as the *_get functions.
 *
 */

int serialq_peek ( int *val )
{

	int success;

	irq_disable(0);

	if(sciq_of)
	{
		success = -2;
	}

	else if(sciq_head==sciq_tail)
	{
		success = 0;
	}

	else
	{
		*val = serialq[sciq_head-sciq_start];
		success = 1;
	}

	irq_enable(0);
	return success;

}


/*
 * read and reset the condition code
 *
 * returns the condition code set by the interrupt routine.
 *
 */

int serialq_cc_read()
{
	return sciq_of;
}
void serial_cc_reset()
{
	sciq_of = 0;
}




/* private subroutine for serialq */
void read_val( int *val )
{

	*val = serialq[sciq_head-sciq_start]; 

	sciq_head++;

	if(sciq_head==sciq_stop)	sciq_head = sciq_start;

}
